"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TscWatchClient = void 0;
const child_process_1 = require("child_process");
const events_1 = require("events");
const path_1 = require("path");
class TscWatchClient extends events_1.EventEmitter {
    start(...args) {
        const tscWatch = require.resolve((0, path_1.join)(__dirname, 'dist', 'lib', 'tsc-watch'));
        this.tsc = (0, child_process_1.fork)(tscWatch, args, { stdio: 'inherit' });
        this.tsc.on('message', (msg) => {
            this.emit(...deserializeTscMessage(msg));
        });
        this.tsc.on('exit', (code, signal) => {
            this.emit('exit', code, signal);
        });
    }
    kill() {
        if (this.tsc && this.tsc.kill) {
            this.tsc.kill();
        }
        this.removeAllListeners();
    }
    runOnCompilationStartedCommand() {
        if (this.tsc) {
            this.tsc.send('run-on-compilation-started-command');
        }
    }
    runOnCompilationCompleteCommand() {
        if (this.tsc) {
            this.tsc.send('run-on-compilation-complete-command');
        }
    }
    runOnFirstSuccessCommand() {
        if (this.tsc) {
            this.tsc.send('run-on-first-success-command');
        }
    }
    runOnFailureCommand() {
        if (this.tsc) {
            this.tsc.send('run-on-failure-command');
        }
    }
    runOnSuccessCommand() {
        if (this.tsc) {
            this.tsc.send('run-on-success-command');
        }
    }
}
exports.TscWatchClient = TscWatchClient;
function deserializeTscMessage(strMsg) {
    const indexOfSeparator = strMsg.indexOf(':');
    if (indexOfSeparator === -1) {
        return [strMsg];
    }
    return [strMsg.substring(0, indexOfSeparator), strMsg.substring(indexOfSeparator + 1)];
}
