#!/usr/bin/env node
"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_cleanup_1 = __importStar(require("node-cleanup"));
const cross_spawn_1 = __importDefault(require("cross-spawn"));
const runner_1 = require("./runner");
const args_manager_1 = require("./args-manager");
const stdout_manipulator_1 = require("./stdout-manipulator");
const readline_1 = require("readline");
let firstTime = true;
let firstSuccessKiller = null;
let successKiller = null;
let failureKiller = null;
let compilationStartedKiller = null;
let compilationCompleteKiller = null;
const { onFirstSuccessCommand, onSuccessCommand, onFailureCommand, onCompilationStarted, onCompilationComplete, maxNodeMem, noColors, noClear, silent, compiler, args, } = (0, args_manager_1.extractArgs)(process.argv);
function killProcesses(killAll) {
    return Promise.all([
        killAll && firstSuccessKiller ? firstSuccessKiller() : null,
        successKiller ? successKiller() : null,
        failureKiller ? failureKiller() : null,
        compilationStartedKiller ? compilationStartedKiller() : null,
        compilationCompleteKiller ? compilationCompleteKiller() : null,
    ]);
}
function runOnCompilationStarted() {
    if (onCompilationStarted) {
        compilationStartedKiller = (0, runner_1.run)(onCompilationStarted);
    }
}
function runOnCompilationComplete() {
    if (onCompilationComplete) {
        compilationCompleteKiller = (0, runner_1.run)(onCompilationComplete);
    }
}
function runOnFailureCommand() {
    if (onFailureCommand) {
        failureKiller = (0, runner_1.run)(onFailureCommand);
    }
}
function runOnFirstSuccessCommand() {
    if (onFirstSuccessCommand) {
        firstSuccessKiller = (0, runner_1.run)(onFirstSuccessCommand);
    }
}
function runOnSuccessCommand() {
    if (onSuccessCommand) {
        successKiller = (0, runner_1.run)(onSuccessCommand);
    }
}
function getTscPath() {
    let tscBin;
    try {
        return require.resolve(compiler);
    }
    catch (e) {
        if (e.code === 'MODULE_NOT_FOUND') {
            console.error(e.message);
            process.exit(9);
        }
        throw e;
    }
}
function spawnTsc({ maxNodeMem }, args) {
    const nodeArgs = [];
    if (maxNodeMem) {
        nodeArgs.push(`--max_old_space_size=${maxNodeMem}`);
    }
    const tscBin = getTscPath();
    nodeArgs.push(tscBin);
    nodeArgs.push(...args);
    return (0, cross_spawn_1.default)('node', nodeArgs);
}
let compilationErrorSinceStart = false;
const tscProcess = spawnTsc({ maxNodeMem }, args);
if (!tscProcess.stdout) {
    throw new Error('Unable to read Typescript stdout');
}
const rl = (0, readline_1.createInterface)({ input: tscProcess.stdout });
rl.on('line', function (input) {
    if (noClear) {
        input = (0, stdout_manipulator_1.deleteClear)(input);
    }
    const line = (0, stdout_manipulator_1.manipulate)(input);
    if (!silent) {
        (0, stdout_manipulator_1.print)(noColors, noClear, line);
    }
    const state = (0, stdout_manipulator_1.detectState)(line);
    const compilationStarted = state.compilationStarted;
    const compilationError = state.compilationError;
    const compilationComplete = state.compilationComplete;
    compilationErrorSinceStart =
        (!compilationStarted && compilationErrorSinceStart) || compilationError;
    if (state.fileEmitted !== null) {
        Signal.emitFile(state.fileEmitted);
    }
    if (compilationStarted) {
        killProcesses(false).then(() => {
            runOnCompilationStarted();
            Signal.emitStarted();
        });
    }
    if (compilationComplete) {
        killProcesses(false).then(() => {
            runOnCompilationComplete();
            if (compilationErrorSinceStart) {
                Signal.emitFail();
                runOnFailureCommand();
            }
            else {
                if (firstTime) {
                    firstTime = false;
                    Signal.emitFirstSuccess();
                    runOnFirstSuccessCommand();
                }
                Signal.emitSuccess();
                runOnSuccessCommand();
            }
        });
    }
});
if (typeof process.on === 'function') {
    process.on('message', (msg) => {
        switch (msg) {
            case 'run-on-compilation-started-command':
                if (compilationStartedKiller) {
                    compilationStartedKiller().then(runOnCompilationStarted);
                }
                break;
            case 'run-on-compilation-complete-command':
                if (compilationCompleteKiller) {
                    compilationCompleteKiller().then(runOnCompilationComplete);
                }
                break;
            case 'run-on-first-success-command':
                if (firstSuccessKiller) {
                    firstSuccessKiller().then(runOnFirstSuccessCommand);
                }
                break;
            case 'run-on-failure-command':
                if (failureKiller) {
                    failureKiller().then(runOnFailureCommand);
                }
                break;
            case 'run-on-success-command':
                if (successKiller) {
                    successKiller().then(runOnSuccessCommand);
                }
                break;
            default:
                console.log('Unknown message', msg);
        }
    });
}
const sendSignal = (msg) => {
    if (process.send) {
        process.send(msg);
    }
};
const Signal = {
    emitStarted: () => sendSignal('started'),
    emitFirstSuccess: () => sendSignal('first_success'),
    emitSuccess: () => sendSignal('success'),
    emitFail: () => sendSignal('compile_errors'),
    emitFile: (path) => sendSignal(`file_emitted:${path}`),
};
(0, node_cleanup_1.default)((_exitCode, signal) => {
    if (signal) {
        tscProcess.kill(signal);
    }
    killProcesses(true).then(() => process.exit());
    // don't call cleanup handler again
    (0, node_cleanup_1.uninstall)();
    return false;
});
